@@ -19,11 +19,14 @@
  */
 
 #include "glomconversions.h"
+#include "../connectionpool.h"
+#include "../utils.h"
 #include <sstream> //For stringstream
 
 #include <locale>     // for locale, time_put
 #include <ctime>     // for struct tm
 #include <iostream>   // for cout, endl
+#include <iomanip>
 
 
 Glib::ustring GlomConversions::format_time(const tm& tm_data)
@@ -116,22 +119,46 @@ Glib::ustring GlomConversions::format_tm(const tm& tm_data, const std::locale& l
   */
 }
 
-Glib::ustring GlomConversions::get_text_for_gda_value(Field::glom_field_type glom_type, const Gnome::Gda::Value& value)
+namespace //anonymous
 {
-  return get_text_for_gda_value(glom_type, value, std::locale("") /* the user's current locale */ ); //Get the current locale.
+
+class numpunct_thousands_separator: public std::numpunct<char>
+{
+  //Override
+  std::string do_grouping() const
+  {
+    return "\3";
+  };
+};
+
+class numpunct_no_thousands_separator: public std::numpunct<char>
+{
+  //Override
+  std::string do_grouping() const
+  {
+    return "";
+  };
+};
+
+} //anonymous namespace
+
+Glib::ustring GlomConversions::get_text_for_gda_value(Field::glom_field_type glom_type, const Gnome::Gda::Value& value, const NumericFormat& numeric_format)
+{
+  return get_text_for_gda_value(glom_type, value, std::locale("") /* the user's current locale */, numeric_format); //Get the current locale.
 }
 
-Glib::ustring GlomConversions::get_text_for_gda_value(Field::glom_field_type glom_type, const Gnome::Gda::Value& value, const std::locale& locale, bool iso_format)
+
+Glib::ustring GlomConversions::get_text_for_gda_value(Field::glom_field_type glom_type, const Gnome::Gda::Value& value, const std::locale& locale, const NumericFormat& numeric_format, bool iso_format)
 {
   if(value.get_value_type() == Gnome::Gda::VALUE_TYPE_NULL) //The type can be null for any of the actual field types.
   {
     return "";
   }
 
-  if(glom_type == Field::TYPE_DATE)
+  if( (glom_type == Field::TYPE_DATE) && (value.get_value_type() == Gnome::Gda::VALUE_TYPE_DATE))
   {
     Gnome::Gda::Date gda_date = value.get_date();
-    
+
     tm the_c_time = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
     the_c_time.tm_year = gda_date.year - 1900; //C years start are the AD year - 1900. So, 01 is 1901.
     the_c_time.tm_mon = gda_date.month - 1; //C months start at 0.
@@ -145,7 +172,7 @@ Glib::ustring GlomConversions::get_text_for_gda_value(Field::glom_field_type glo
     return date.format_string("%x"); //%x means "is replaced by the locale's appropriate date representation".
     */
   }
-  else if(glom_type == Field::TYPE_TIME)
+  else if((glom_type == Field::TYPE_TIME) && (value.get_value_type() == Gnome::Gda::VALUE_TYPE_TIME))
   {
     Gnome::Gda::Time gda_time = value.get_time();
 
@@ -156,7 +183,7 @@ Glib::ustring GlomConversions::get_text_for_gda_value(Field::glom_field_type glo
 
     return format_time(the_c_time, locale, iso_format);
   }
-  else if(glom_type == Field::TYPE_NUMERIC)
+  else if( (glom_type == Field::TYPE_NUMERIC) && (value.get_value_type() == Gnome::Gda::VALUE_TYPE_NUMERIC))
   {
     const GdaNumeric* gda_numeric = value.get_numeric();
     std::string text_in_c_locale;
@@ -167,24 +194,46 @@ Glib::ustring GlomConversions::get_text_for_gda_value(Field::glom_field_type glo
     std::stringstream the_stream;
     the_stream.imbue( std::locale::classic() ); //The C locale.
     the_stream.str(text_in_c_locale); //Avoid using << because Glib::ustinrg does implicit character conversion with that.
-   
+
     double number = 0;
     the_stream >> number;
-  
-    //Get the locale-specific text representation:
+
+    //Get the locale-specific text representation, in the required format:
     std::stringstream another_stream;
     another_stream.imbue(locale); //Tell it to parse stuff as per this locale.
+
+    //Numeric formatting:
+    if(!iso_format)
+    {
+
+      if(!numeric_format.m_use_thousands_separator)
+      {
+        std::locale locale_modified(locale, new numpunct_no_thousands_separator()); //The locale takes ownership.
+        another_stream.imbue(locale_modified);
+      }
+
+      if(numeric_format.m_decimal_places_restricted)
+      {
+        another_stream << std::fixed;
+        another_stream << std::setprecision(numeric_format.m_decimal_places); //precision means number of decimal places when using std::fixed.
+      }
+
+      if(!(numeric_format.m_currency_symbol.empty()))
+      {
+        another_stream << numeric_format.m_currency_symbol << " ";
+      }
+    }
+
     another_stream << number;
-    Glib::ustring text;
-    text = another_stream.str();  //Avoid using << because Glib::ustinrg does implicit character conversion with that.
-     
+    Glib::ustring text = another_stream.str();  //Avoid using << because Glib::ustring does implicit character conversion with that.
+
     if(locale == std::locale("") /* The user's current locale */)
     {
       //Converts from the user's current locale to utf8. I would prefer a generic conversion from any locale,
       // but there is no such function, and it's hard to do because I don't know how to get the encoding name from the std::locale()
       text = Glib::locale_to_utf8(text); 
     }
-    
+
     return text; //Do something like Glib::locale_to_utf(), but with the specified locale instead of the current locale.
   }
   else
@@ -193,10 +242,40 @@ Glib::ustring GlomConversions::get_text_for_gda_value(Field::glom_field_type glo
   }
 }
 
+Gnome::Gda::Value GlomConversions::parse_value(double number)
+{
+  //This is just a way to get a NUMERIC Gda::Value from a numeric type:
+  //Try to parse the inputted number, according to the current locale.
+
+  GdaNumeric gda_numeric = {0, 0, 0};
+
+  //Then generate a canonical representation of the number:
+  std::stringstream clocale_stream;
+  clocale_stream.imbue( std::locale::classic() ); //The C locale.
+  clocale_stream << number;
+  Glib::ustring number_canonical_text = clocale_stream.str(); //Avoid << because it does implicit character conversion (though that might not be a problem here. Not sure). murrayc
+
+  //TODO: What about the precision and width values?
+  /* From the postgres docs:
+  *  The scale of a numeric is the count of decimal digits in the fractional part, to the right of the decimal point.
+  * The precision of a numeric is the total count of significant digits in the whole number, that is, the number of digits to both sides of the decimal point.
+  * So the number 23.5141 has a precision of 6 and a scale of 4. Integers can be considered to have a scale of zero.
+  */
+  gda_numeric.number = g_strdup(number_canonical_text.c_str());
+
+  return Gnome::Gda::Value(&gda_numeric);
+}
+
 Gnome::Gda::Value GlomConversions::parse_value(Field::glom_field_type glom_type, const Glib::ustring& text, bool& success, bool iso_format)
+{
+  NumericFormat ignore_format;
+  return parse_value(glom_type, text, ignore_format, success, iso_format);
+}
+
+Gnome::Gda::Value GlomConversions::parse_value(Field::glom_field_type glom_type, const Glib::ustring& text, const NumericFormat& numeric_format, bool& success, bool iso_format)
 {
   std::locale the_locale = (iso_format ? std::locale::classic() :  std::locale("") /* The user's current locale */);
-  
+
   //Put a NULL in the database for empty dates, times, and numerics, because 0 would be an actual value.
   //But we use "" for strings, because the distinction between NULL and "" would not be clear to users.
   if(text.empty())
@@ -233,21 +312,37 @@ Gnome::Gda::Value GlomConversions::parse_value(Field::glom_field_type glom_type,
   }
   else if(glom_type == Field::TYPE_NUMERIC)
   {
+    Glib::ustring text_to_parse = GlomUtils::trim_whitespace(text);
+
+    if(!(numeric_format.m_currency_symbol.empty()))
+    {
+      //Remove the currency symbol:
+      const Glib::ustring prefix = text_to_parse.substr(0, numeric_format.m_currency_symbol.size());
+      if(text_to_parse.substr(0, numeric_format.m_currency_symbol.size()) == numeric_format.m_currency_symbol)
+      {
+        text_to_parse = text_to_parse.substr(numeric_format.m_currency_symbol.size());
+        text_to_parse = GlomUtils::trim_whitespace(text_to_parse); //remove any whitespace between the currency symbol and the number.
+      }
+    }
+
+
+    //text_to_parse = Base_DB::string_trim(text_to_parse, " ");
+
     //Try to parse the inputted number, according to the current locale.
     std::stringstream the_stream;
     the_stream.imbue( the_locale ); //Parse it as per the current locale.
-    the_stream.str(text); //Avoid << because it does implicit character conversion (though that might not be a problem here. Not sure). murrayc
+    the_stream.str(text_to_parse); //Avoid << because it does implicit character conversion (though that might not be a problem here. Not sure). murrayc
     double the_number = 0;
     the_stream >> the_number;  //TODO: Does this throw any exception if the text is an invalid time?
 
     GdaNumeric gda_numeric = {0, 0, 0};
-    
+
     //Then generate a canonical representation of the number:
-    Glib::ustring number_canonical_text;
+
     std::stringstream clocale_stream;
     clocale_stream.imbue( std::locale::classic() ); //The C locale.
     clocale_stream << the_number;
-    number_canonical_text = clocale_stream.str(); //Avoid << because it does implicit character conversion (though that might not be a problem here. Not sure). murrayc
+    Glib::ustring  number_canonical_text = clocale_stream.str(); //Avoid << because it does implicit character conversion (though that might not be a problem here. Not sure). murrayc
 
     //TODO: What about the precision and width values?
     /* From the postgres docs:
@@ -341,15 +436,14 @@ tm GlomConversions::parse_date(const Glib::ustring& text, const std::locale& loc
       success = false;
     }
   }
-  
-    
+
   //Prevent some nonsense values:
   if(the_c_time.tm_mday == 0)
     the_c_time.tm_mday = 1;
-    
+
   if(the_c_time.tm_mon == 0)
       the_c_time.tm_mon = 1;
-      
+
   return the_c_time;
 }
 
@@ -477,4 +571,33 @@ Gnome::Gda::Value GlomConversions::get_empty_value(Field::glom_field_type field_
   }
 }
 
-  
+Gnome::Gda::Value GlomConversions::get_example_value(Field::glom_field_type field_type)
+{
+  switch(field_type)
+  {
+    case(Field::TYPE_BOOLEAN):
+      return Gnome::Gda::Value(true);
+    case(Field::TYPE_DATE):
+    {
+      bool success = false;
+      return parse_value(field_type, "01/02/03", success, true /* iso_format */);
+    }
+    case(Field::TYPE_NUMERIC):
+    {
+      bool success = false;
+      return parse_value(field_type, "1", success, true /* iso_format */);
+    }
+    case(Field::TYPE_TEXT):
+      return Gnome::Gda::Value( Glib::ustring("example") ); //Use an empty string instead of a null for text fields, because the distinction is confusing for users, and gives no advantages.
+    case(Field::TYPE_TIME):
+    {
+      bool success = false;
+      return parse_value(field_type, "01:02", success, true /* iso_format */);
+    }
+    default:
+      return Gnome::Gda::Value();
+  }
+}
+
+
+
