@@ -23,10 +23,9 @@
 #include "../data_structure/relationship.h"
 #include "../data_structure/glomconversions.h"
 #include "dialog_layout_details.h"
+#include "../utils.h"
 #include <bakery/App/App_Gtk.h> //For util_bold_message().
 #include "../python_embed/glom_python.h"
-#include <libgnomevfsmm.h>
-#include <libgnome/gnome-url.h>
 #include <sstream> //For stringstream
 #include <glibmm/i18n.h>
 
@@ -61,7 +60,7 @@ Box_Data_Details::Box_Data_Details(bool bWithNavButtons /* = true */)
   m_FlowTable.set_columns_count(1); //Sub-groups will have multiple columns (by default, there is one sub-group, with 2 columns).
   m_FlowTable.set_padding(6);
 
-  m_strHint = _("When you change the data in a field the database is updated immediately.\n Click [New] to add a new record.\n Leave automatic ID fields empty - they will be filled for you.");
+  //m_strHint = _("When you change the data in a field the database is updated immediately.\n Click [New] to add a new record.\n Leave automatic ID fields empty - they will be filled for you.");
 
 
   //m_Paned.set_border_width(6);
@@ -129,24 +128,24 @@ Gnome::Gda::Value Box_Data_Details::get_primary_key_value() const
   return m_primary_key_value;
 }
 
-void Box_Data_Details::init_db_details(const Glib::ustring& strTableName, const Gnome::Gda::Value& primary_key_value)
+bool Box_Data_Details::init_db_details(const Glib::ustring& strTableName, const Gnome::Gda::Value& primary_key_value)
 {
   m_primary_key_value = primary_key_value;
 
   get_field_primary_key_for_table(strTableName, m_field_primary_key);
 
-  Box_Data::init_db_details(strTableName); //Calls create_layout(), then fill_from_database()
+  return Box_Data::init_db_details(strTableName); //Calls create_layout(), then fill_from_database()
 }
 
-void Box_Data_Details::refresh_data_from_database(const Gnome::Gda::Value& primary_key_value)
+bool Box_Data_Details::refresh_data_from_database(const Gnome::Gda::Value& primary_key_value)
 {
   m_primary_key_value = primary_key_value;
-  fill_from_database();
+  return fill_from_database();
 }
 
-void Box_Data_Details::refresh_data_from_database_blank()
+bool Box_Data_Details::refresh_data_from_database_blank()
 {
-  refresh_data_from_database( Gnome::Gda::Value() );
+  return refresh_data_from_database( Gnome::Gda::Value() );
 }
 
 void Box_Data_Details::create_layout()
@@ -174,18 +173,21 @@ void Box_Data_Details::create_layout()
 
 }
 
-void Box_Data_Details::fill_from_database()
+bool Box_Data_Details::fill_from_database()
 {
+  bool bResult = false;
+
   Bakery::BusyCursor(*get_app_window());
 
   try
   {
+
     //TODO: This should keep the connection open, so we don't need to 
     //reconnect many times..
     sharedptr<SharedConnection> sharedconnection = connect_to_server();
 
 
-    Box_Data::fill_from_database();
+    bResult = Box_Data::fill_from_database();
 
     m_FieldsShown = get_fields_to_show();
     type_vecLayoutFields fieldsToGet = m_FieldsShown;
@@ -202,8 +204,8 @@ void Box_Data_Details::fill_from_database()
       if(table_privs.m_view)
       {
         //Add extra possibly-non-visible columns that we need:
-        LayoutItem_Field layout_item;
-        layout_item.m_field = m_field_primary_key;
+        sharedptr<LayoutItem_Field> layout_item(new LayoutItem_Field);
+        layout_item->m_field = m_field_primary_key;
         fieldsToGet.push_back(layout_item);
 
         //g_warning("primary_key name = %s", m_field_primary_key.get_name().c_str());
@@ -230,14 +232,18 @@ void Box_Data_Details::fill_from_database()
             //Get field values to show:
             for(int i = 0; i < cols_count; ++i)
             {
-              const LayoutItem_Field& layout_item = fieldsToGet[i];
+              sharedptr<LayoutItem_Field> layout_item = fieldsToGet[i];
 
               //Field value:
               Gnome::Gda::Value value = result->get_value_at(i, row_number);
-              m_FlowTable.set_field_value(layout_item, value);
+              m_FlowTable.set_field_value(*layout_item, value);
             }
           }
         }
+        else
+        {
+          bResult = false; //There were no records.
+        }
       }
     } //if(!fieldsToGet.empty())
 
@@ -251,7 +257,10 @@ void Box_Data_Details::fill_from_database()
   catch(std::exception& ex)
   {
     handle_error(ex);
+    bResult = false;
   }
+
+  return bResult;
 }
 
 void Box_Data_Details::on_button_new()
@@ -356,7 +365,7 @@ void Box_Data_Details::on_related_user_requested_details(Gnome::Gda::Value key_v
 
 void Box_Data_Details::recalculate_fields_for_related_records(const Glib::ustring& relationship_name)
 {
- //TODO: Get (recursively) the list of fields to recalcualte, sort them in order of dependency, then calculate them all.
+  m_FieldsCalculationInProgress.clear();
 
   const Gnome::Gda::Value primary_key_value = get_primary_key_value();
 
@@ -369,20 +378,19 @@ void Box_Data_Details::recalculate_fields_for_related_records(const Glib::ustrin
     Field::type_list_strings::const_iterator iterFind = std::find(triggered_by.begin(), triggered_by.end(), relationship_name);
     if(iterFind != triggered_by.end()) //If it was found
     {
-       //recalculate:
-       const type_map_fields field_values = get_record_field_values(primary_key_value);
-       Gnome::Gda::Value value = glom_evaluate_python_function_implementation(field.get_glom_type(), field.get_calculation(), field_values,
-          get_document(), m_strTableName);
+      const Field& field = *iter;
 
-      //show it, if it's on the layout:
+      calculate_field(field, m_field_primary_key, primary_key_value); //And any dependencies.
+
+      //Calculate anything that depends on this.
       LayoutItem_Field layout_item;
       layout_item.m_field = field;
-      set_entered_field_data(layout_item, value);
 
-      //Add it to the database (even if it is not shown in the layout)
-      set_field_value_in_database(layout_item, value, m_field_primary_key, primary_key_value);
+      do_calculations(layout_item, m_field_primary_key, primary_key_value, false /* recurse, reusing m_FieldsCalculationInProgress */);
     }
   }
+
+   m_FieldsCalculationInProgress.clear();
 }
 
 void Box_Data_Details::on_related_record_added(Gnome::Gda::Value /* strKeyValue */, Glib::ustring /* strFromKeyName */)
@@ -644,12 +652,12 @@ void Box_Data_Details::refresh_related_fields(const Gtk::TreeModel::iterator& /*
         for(guint uiCol = 0; uiCol < cols_count; uiCol++)
         {
           const Gnome::Gda::Value value = result->get_value_at(uiCol, 0 /* row */);
-          const LayoutItem_Field& layout_item = *iterFields;
+          sharedptr<LayoutItem_Field> layout_item = *iterFields;
 
           //g_warning("list fill: field_name=%s", iterFields->get_name().c_str());
           //g_warning("  value_as_string=%s", value.to_string().c_str());
 
-          m_FlowTable.set_field_value(layout_item, value);
+          m_FlowTable.set_field_value(*layout_item, value);
 
           ++iterFields;
         }
@@ -722,7 +730,7 @@ void Box_Data_Details::print_layout_group(xmlpp::Element* node_parent, const Lay
       {
         xmlpp::Element* nodeField = nodeChildGroup->add_child("field");
 
-        nodeField->set_attribute("title", pLayoutField->m_field.get_title_or_name());
+        nodeField->set_attribute("title", pLayoutField->get_title_or_name());
 
         Gnome::Gda::Value value = m_FlowTable.get_field_value(*pLayoutField);
         Glib::ustring text_representation = GlomConversions::get_text_for_gda_value(pLayoutField->m_field.get_glom_type(), value,
@@ -795,43 +803,6 @@ void Box_Data_Details::print_layout()
       print_layout_group(nodeParent, layout_group);
     }
 
-
-    //Use libxslt to convert the XML to HTML:
-    Glib::ustring result = xslt_process(*pDocument, GLOM_XSLTDIR "print_details_to_html.xsl");
-    std::cout << "After xslt: " << result << std::endl;
-
-    //Save it to a temporary file and show it in a browser:
-    //TODO: This actually shows it in gedit.
-    const Glib::ustring temp_uri = "file:///tmp/glom_printout.html";
-    std::cout << "temp_uri=" << temp_uri << std::endl;
-
-    Gnome::Vfs::Handle write_handle;
-
-    try
-    {
-      //0660 means "this user and his group can read and write this non-executable file".
-      //The 0 prefix means that this is octal.
-      write_handle.create(temp_uri, Gnome::Vfs::OPEN_WRITE, false, 0660 /* leading zero means octal */);
-    }
-    catch(const Gnome::Vfs::exception& ex)
-    {
-      // If the operation was not successful, print the error and abort
-      return; // print_error(ex, output_uri_string);
-    }
-
-    try
-    {
-      //Write the data to the output uri
-      /* GnomeVFSFileSize bytes_written = */ write_handle.write(result.data(), result.bytes());
-    }
-    catch(const Gnome::Vfs::exception& ex)
-    {
-      // If the operation was not successful, print the error and abort
-      return; //print_error(ex, output_uri_string);
-    }
-
-    //Use the GNOME browser:
-    GError* error = 0;
-    gnome_url_show(temp_uri.c_str(), &error); //This is in libgnome.
+    GlomUtils::transform_and_open(*pDocument, "print_details_to_html.xsl");
   }
 }
