@@ -20,9 +20,11 @@
 
 #include "dialog_layout_details.h"
 #include "dialog_choose_field.h"
+#include "../layout_item_dialogs/dialog_field_layout.h"
 #include "dialog_choose_relationship.h"
 //#include <libgnome/gnome-i18n.h>
-#include <libintl.h>
+#include <bakery/App/App_Gtk.h> //For util_bold_message().
+#include <glibmm/i18n.h>
 #include <sstream> //For stringstream
 
 Dialog_Layout_Details::Dialog_Layout_Details(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade)
@@ -34,6 +36,7 @@ Dialog_Layout_Details::Dialog_Layout_Details(BaseObjectType* cobject, const Glib
   m_button_field_add_group(0),
   m_button_add_related(0),
   m_button_field_delete(0),
+  m_button_field_formatting(0),
   m_button_field_edit(0),
   m_label_table_name(0)
 {
@@ -45,7 +48,7 @@ Dialog_Layout_Details::Dialog_Layout_Details(BaseObjectType* cobject, const Glib
     m_treeview_fields->set_reorderable();
     m_treeview_fields->enable_model_drag_source();
     m_treeview_fields->enable_model_drag_dest();
-    
+
     m_model_items = TreeStore_Layout::create();
     m_treeview_fields->set_model(m_model_items);
 
@@ -53,7 +56,7 @@ Dialog_Layout_Details::Dialog_Layout_Details(BaseObjectType* cobject, const Glib
     // Use set_cell_data_func() to give more control over the cell attributes depending on the row:
 
     //Name column:
-    Gtk::TreeView::Column* column_name = Gtk::manage( new Gtk::TreeView::Column(gettext("Name")) );
+    Gtk::TreeView::Column* column_name = Gtk::manage( new Gtk::TreeView::Column(_("Name")) );
     m_treeview_fields->append_column(*column_name);
 
     Gtk::CellRendererText* renderer_name = Gtk::manage(new Gtk::CellRendererText);
@@ -67,7 +70,7 @@ Dialog_Layout_Details::Dialog_Layout_Details(BaseObjectType* cobject, const Glib
 
 
     //Title column:
-    Gtk::TreeView::Column* column_title = Gtk::manage( new Gtk::TreeView::Column(gettext("Title")) );
+    Gtk::TreeView::Column* column_title = Gtk::manage( new Gtk::TreeView::Column(_("Title")) );
     m_treeview_fields->append_column(*column_title);
 
     Gtk::CellRendererText* renderer_title = Gtk::manage(new Gtk::CellRendererText);
@@ -80,7 +83,7 @@ Dialog_Layout_Details::Dialog_Layout_Details(BaseObjectType* cobject, const Glib
 
 
     //Columns-count column:
-    Gtk::TreeView::Column* column_count = Gtk::manage( new Gtk::TreeView::Column(gettext("Columns Count")) );
+    Gtk::TreeView::Column* column_count = Gtk::manage( new Gtk::TreeView::Column(_("Columns Count")) );
     m_treeview_fields->append_column(*column_count);
 
     Gtk::CellRendererText* renderer_count = Gtk::manage(new Gtk::CellRendererText);
@@ -110,6 +113,9 @@ Dialog_Layout_Details::Dialog_Layout_Details(BaseObjectType* cobject, const Glib
   refGlade->get_widget("button_field_delete", m_button_field_delete);
   m_button_field_delete->signal_clicked().connect( sigc::mem_fun(*this, &Dialog_Layout_Details::on_button_field_delete) );
 
+  refGlade->get_widget("button_formatting", m_button_field_formatting);
+  m_button_field_formatting->signal_clicked().connect( sigc::mem_fun(*this, &Dialog_Layout_Details::on_button_field_formatting) );
+
   refGlade->get_widget("button_field_add", m_button_field_add);
   m_button_field_add->signal_clicked().connect( sigc::mem_fun(*this, &Dialog_Layout_Details::on_button_field_add) );
 
@@ -143,6 +149,7 @@ void Dialog_Layout_Details::fill_group(const Gtk::TreeModel::iterator& iter, Lay
     for(Gtk::TreeModel::iterator iterChild = row.children().begin(); iterChild != row.children().end(); ++iterChild)
     {
       Gtk::TreeModel::Row rowChild = *iterChild;
+
       if(rowChild[m_model_items->m_columns.m_col_type] == TreeStore_Layout::TYPE_GROUP)
       {
         //Recurse:
@@ -159,9 +166,18 @@ void Dialog_Layout_Details::fill_group(const Gtk::TreeModel::iterator& iter, Lay
       else if(rowChild[m_model_items->m_columns.m_col_type] == TreeStore_Layout::TYPE_FIELD)
       {
         //Add field:
-        LayoutItem_Field field;
+        LayoutItem_Field field = rowChild[m_model_items->m_columns.m_col_field_formatting]; //TODO: Use _only_ this for fields in future?
+
         field.set_name( rowChild[m_model_items->m_columns.m_col_name] );
-        field.set_table_name(m_table_name);
+
+        field.m_relationship = rowChild[m_model_items->m_columns.m_col_relationship_name];
+
+        //if(!relationship_name.empty())
+        //{
+        //  get_document()->get_table_relationship(m_table_name, field.m_relationship);
+        //}
+
+        field.set_editable( rowChild[m_model_items->m_columns.m_col_editable] );
 
         group.add_item(field);
       }
@@ -191,6 +207,7 @@ void Dialog_Layout_Details::add_group(const Gtk::TreeModel::iterator& parent, co
     row[m_model_items->m_columns.m_col_name] = group.get_name();
     row[m_model_items->m_columns.m_col_columns_count] = group.m_columns_count;
     row[m_model_items->m_columns.m_col_title] = group.m_title;
+    row[m_model_items->m_columns.m_col_editable] = group.get_editable();
 
     //Add the child items:
     LayoutGroup::type_map_const_items items = group.get_items();
@@ -219,7 +236,11 @@ void Dialog_Layout_Details::add_group(const Gtk::TreeModel::iterator& parent, co
             Gtk::TreeModel::iterator iterField = m_model_items->append(iterNewGroup->children());
             Gtk::TreeModel::Row row = *iterField;
             row[m_model_items->m_columns.m_col_type] = TreeStore_Layout::TYPE_FIELD;
+            row[m_model_items->m_columns.m_col_field_formatting] = *field;
             row[m_model_items->m_columns.m_col_name] = field->get_name();
+            row[m_model_items->m_columns.m_col_relationship_name] = field->m_relationship;
+
+            row[m_model_items->m_columns.m_col_editable] = field->get_editable();
           }
         }
       }
@@ -229,7 +250,7 @@ void Dialog_Layout_Details::add_group(const Gtk::TreeModel::iterator& parent, co
   }
 }
 
-void Dialog_Layout_Details::set_document(const Glib::ustring& layout, Document_Glom* document, const Glib::ustring& table_name, const type_vecFields& table_fields)
+void Dialog_Layout_Details::set_document(const Glib::ustring& layout, Document_Glom* document, const Glib::ustring& table_name, const type_vecLayoutFields& table_fields)
 {
   m_modified = false;
 
@@ -244,6 +265,7 @@ void Dialog_Layout_Details::set_document(const Glib::ustring& layout, Document_G
     m_entry_table_title->set_text( document->get_table_title(table_name) );
 
     Document_Glom::type_mapLayoutGroupSequence mapGroups = document->get_data_layout_groups_plus_new_fields(layout, m_table_name);
+    document->fill_layout_field_details(m_table_name, mapGroups); //Update with full field information.
 
     //If no information is stored in the document, then start with something:
 
@@ -254,11 +276,9 @@ void Dialog_Layout_Details::set_document(const Glib::ustring& layout, Document_G
       group.m_columns_count = 2;
 
       guint field_sequence = 1; //0 means no sequence
-      for(type_vecFields::const_iterator iter = table_fields.begin(); iter != table_fields.end(); ++iter)
+      for(type_vecLayoutFields::const_iterator iter = table_fields.begin(); iter != table_fields.end(); ++iter)
       {
-        LayoutItem_Field item;
-        item.set_name(iter->get_name());
-        item.set_table_name(m_table_name);
+        LayoutItem_Field item = *(*iter);
         item.m_sequence = field_sequence;
 
         group.add_item(item, field_sequence);
@@ -320,6 +340,8 @@ void Dialog_Layout_Details::enable_buttons()
       m_button_field_down->set_sensitive(enable_down);
 
       m_button_field_delete->set_sensitive(true);
+
+      m_button_field_formatting->set_sensitive( (*iter)[m_model_items->m_columns.m_col_type] == TreeStore_Layout::TYPE_FIELD);
     }
     else
     {
@@ -327,6 +349,7 @@ void Dialog_Layout_Details::enable_buttons()
       m_button_field_down->set_sensitive(false);
       m_button_field_up->set_sensitive(false);
       m_button_field_delete->set_sensitive(false);
+      m_button_field_formatting->set_sensitive(false);
     }
   }
 
@@ -416,8 +439,8 @@ void Dialog_Layout_Details::on_button_field_add()
 {
   Gtk::TreeModel::iterator parent = get_selected_group_parent();
 
-  Field field;
-  bool test = offer_field_list(field);
+  LayoutItem_Field layout_item;
+  bool test = offer_field_list(layout_item);
   if(test)
   {
     //Add the field details to the layout treeview:
@@ -440,7 +463,9 @@ void Dialog_Layout_Details::on_button_field_add()
     {
       Gtk::TreeModel::Row row = *iter;
       row[m_model_items->m_columns.m_col_type] = TreeStore_Layout::TYPE_FIELD;
-      row[m_model_items->m_columns.m_col_name] = field.get_name();
+      row[m_model_items->m_columns.m_col_name] = layout_item.get_name();
+      row[m_model_items->m_columns.m_col_relationship_name] = layout_item.m_relationship;
+      row[m_model_items->m_columns.m_col_editable] = true; //A sane default.
       //row[m_model_items->m_columns.m_col_title] = field.get_title();
 
       //Scroll to, and select, the new row:
@@ -451,13 +476,15 @@ void Dialog_Layout_Details::on_button_field_add()
       m_treeview_fields->scroll_to_row( Gtk::TreeModel::Path(iter) );
     }
   }
+
+  enable_buttons();
 }
 
 
 bool Dialog_Layout_Details::offer_relationship_list(Relationship& relationship)
 {
   bool result = false;
-  
+
   try
   {
     Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "dialog_choose_relationship");
@@ -467,7 +494,8 @@ bool Dialog_Layout_Details::offer_relationship_list(Relationship& relationship)
 
     if(dialog)
     {
-      dialog->set_document(m_document, m_table_name);
+      dialog->set_document(get_document(), m_table_name);
+      dialog->set_transient_for(*this);
       int response = dialog->run();
       dialog->hide();
       if(response == Gtk::RESPONSE_OK)
@@ -487,7 +515,7 @@ bool Dialog_Layout_Details::offer_relationship_list(Relationship& relationship)
   return result;
 }
 
-bool Dialog_Layout_Details::offer_field_list(Field& field)
+bool Dialog_Layout_Details::offer_field_list(LayoutItem_Field& field)
 {
   bool result = false;
 
@@ -500,7 +528,8 @@ bool Dialog_Layout_Details::offer_field_list(Field& field)
 
     if(dialog)
     {
-      dialog->set_document(m_document, m_table_name, field);
+      dialog->set_document(get_document(), m_table_name, field);
+      dialog->set_transient_for(*this);
       int response = dialog->run();
       if(response == Gtk::RESPONSE_OK)
       {
@@ -519,6 +548,41 @@ bool Dialog_Layout_Details::offer_field_list(Field& field)
   return result;
 }
 
+bool Dialog_Layout_Details::offer_field_layout(LayoutItem_Field& field)
+{
+  bool result = false;
+
+  try
+  {
+    Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "dialog_layout_field_properties");
+
+    Dialog_FieldLayout* dialog = 0;
+    refXml->get_widget_derived("dialog_layout_field_properties", dialog);
+
+    if(dialog)
+    {
+      add_view(dialog); //Give it access to the document.
+      dialog->set_field(field, m_table_name);
+      dialog->set_transient_for(*this);
+      int response = dialog->run();
+      if(response == Gtk::RESPONSE_OK)
+      {
+        //Get the chosen field:
+        result = dialog->get_field_chosen(field);
+      }
+
+      remove_view(dialog);
+      delete dialog;
+    }
+  }
+  catch(const Gnome::Glade::XmlError& ex)
+  {
+    std::cerr << ex.what() << std::endl;
+  }
+
+  return result;
+}
+
 void Dialog_Layout_Details::on_button_add_related()
 {
   Gtk::TreeModel::iterator parent = get_selected_group_parent();
@@ -548,6 +612,7 @@ void Dialog_Layout_Details::on_button_add_related()
       Gtk::TreeModel::Row row = *iter;
       row[m_model_items->m_columns.m_col_type] = TreeStore_Layout::TYPE_PORTAL;
       row[m_model_items->m_columns.m_col_relationship] = relationship.get_name();
+      row[m_model_items->m_columns.m_col_editable] = true; //A sane default.
       //row[m_model_items->m_columns.m_col_title] = field.get_title();
 
       //Scroll to, and select, the new row:
@@ -558,6 +623,8 @@ void Dialog_Layout_Details::on_button_add_related()
       m_treeview_fields->scroll_to_row( Gtk::TreeModel::Path(iter) );
     }
   }
+
+  enable_buttons();
 }
 
 Gtk::TreeModel::iterator Dialog_Layout_Details::get_selected_group_parent() const
@@ -610,6 +677,7 @@ void Dialog_Layout_Details::on_button_field_add_group()
     row[m_model_items->m_columns.m_col_type] = TreeStore_Layout::TYPE_GROUP;
     row[m_model_items->m_columns.m_col_name] = "Untitled group";
     row[m_model_items->m_columns.m_col_columns_count] = 1;
+    row[m_model_items->m_columns.m_col_editable] = true; //A sane default.
 
     //Scroll to, and select, the new row:
     Glib::RefPtr<Gtk::TreeSelection> refTreeSelection = m_treeview_fields->get_selection();
@@ -620,6 +688,58 @@ void Dialog_Layout_Details::on_button_field_add_group()
 
     m_modified = true;
   }
+
+  enable_buttons();
+}
+
+void Dialog_Layout_Details::on_button_field_formatting()
+{
+  //TODO: Abstract this into the base class:
+
+  //Get the selected item:
+  Glib::RefPtr<Gtk::TreeSelection> refTreeSelection = m_treeview_fields->get_selection();
+  if(refTreeSelection)
+  {
+    Gtk::TreeModel::iterator iter = refTreeSelection->get_selected();
+    if(iter)
+    {
+      Gtk::TreeModel::Row row = *iter;
+
+      try
+      {
+        Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "dialog_layout_field_properties");
+
+        Dialog_FieldLayout* dialog = 0;
+        refXml->get_widget_derived("dialog_layout_field_properties", dialog);
+
+        if(dialog)
+        {
+          add_view(dialog); //Give it access to the document.
+
+          LayoutItem_Field field = row[m_model_items->m_columns.m_col_field_formatting];
+
+          dialog->set_field(field, m_table_name);
+          dialog->set_transient_for(*this);
+          int response = dialog->run();
+          dialog->hide();
+          if(response == Gtk::RESPONSE_OK)
+          {
+            //Get the chosen field:
+            bool test = dialog->get_field_chosen(field);
+            if(test)
+              row[m_model_items->m_columns.m_col_field_formatting] = field;
+          }
+
+          remove_view(dialog);
+          delete dialog;
+        }
+      }
+      catch(const Gnome::Glade::XmlError& ex)
+      {
+        std::cerr << ex.what() << std::endl;
+      }
+    }
+  }
 }
 
 void Dialog_Layout_Details::on_button_field_edit()
@@ -644,14 +764,25 @@ void Dialog_Layout_Details::on_button_field_edit()
         }
         case TreeStore_Layout::TYPE_FIELD:
         {
-          Field field;
+          LayoutItem_Field field;
           field.set_name( row[m_model_items->m_columns.m_col_name] ); //Start with this one selected.
+          field.m_relationship = row[m_model_items->m_columns.m_col_relationship_name];
+
+          //if(!relationship_name.empty())
+          //{
+          //  get_document()->get_table_relationship(m_table_name, field.m_relationship);
+          //}
+
+          field.set_editable( row[m_model_items->m_columns.m_col_editable] );
           bool test = offer_field_list(field);
           if(test)
           {
             row[m_model_items->m_columns.m_col_name] = field.get_name();
+            row[m_model_items->m_columns.m_col_relationship_name] = field.m_relationship;
+
+            row[m_model_items->m_columns.m_col_editable] = field.get_editable();
           }
-          
+
           break;
         }
         case TreeStore_Layout::TYPE_PORTAL:
@@ -663,7 +794,7 @@ void Dialog_Layout_Details::on_button_field_edit()
           {
             row[m_model_items->m_columns.m_col_relationship] = relationship.get_name();
           }
-          
+
           break;
         }
       }
@@ -678,8 +809,9 @@ void Dialog_Layout_Details::save_to_document()
   if(m_modified)
   {
     //Set the table name and title:
-    if(m_document)
-      m_document->set_table_title( m_table_name, m_entry_table_title->get_text() );
+    Document_Glom* document = get_document();
+    if(document)
+      document->set_table_title( m_table_name, m_entry_table_title->get_text() );
 
     //Get the data from the TreeView and store it in the document:
 
@@ -707,9 +839,9 @@ void Dialog_Layout_Details::save_to_document()
       }
     }
 
-    if(m_document)
+    if(document)
     {
-      m_document->set_data_layout_groups(m_layout_name, m_table_name, mapGroups);
+      document->set_data_layout_groups(m_layout_name, m_table_name, mapGroups);
       m_modified = false;
     }
   }
@@ -723,7 +855,7 @@ void Dialog_Layout_Details::on_treeview_fields_selection_changed()
 void Dialog_Layout_Details::on_cell_data_name(Gtk::CellRenderer* renderer, const Gtk::TreeModel::iterator& iter)
 {
   //TODO: If we ever use this a real layout tree, then let's add icons for each type.
-  
+
   //Set the view's cell properties depending on the model's data:
   Gtk::CellRendererText* renderer_text = dynamic_cast<Gtk::CellRendererText*>(renderer);
   if(renderer_text)
@@ -740,19 +872,32 @@ void Dialog_Layout_Details::on_cell_data_name(Gtk::CellRenderer* renderer, const
       if(is_group)
       {
         //Make group names bold:
-        markup = "<b>" + row[m_model_items->m_columns.m_col_name] + "</b>";
+        markup = Bakery::App_Gtk::util_bold_message( row[m_model_items->m_columns.m_col_name] );
       }
       else if(is_relationship)
       {
-        markup = gettext("Related: ") + row[m_model_items->m_columns.m_col_relationship];
+        markup = _("Related: ") + row[m_model_items->m_columns.m_col_relationship];
       }
       else
-        markup = row[m_model_items->m_columns.m_col_name];
+      {
+        //It's a field:
+
+        //Indicate that it's a field in another table.
+        const Relationship& rel = row[m_model_items->m_columns.m_col_relationship_name];
+        const Glib::ustring relationship = rel.get_name();
+        if(!relationship.empty())
+          markup = relationship + "::";
+
+        markup += row[m_model_items->m_columns.m_col_name];
+
+        //Just for debugging:
+       // if(!row[m_model_items->m_columns.m_col_editable])
+       //  markup += " *";
+      }
 
       renderer_text->property_markup() = markup;
 
-      renderer_text->property_editable() = row[m_model_items->m_columns.m_col_type] != TreeStore_Layout::TYPE_PORTAL; //Portals can only be edited via the Edit button.
- 
+      renderer_text->property_editable() = false; //Names can never be edited.
     }
   }
 }
