@@ -26,6 +26,7 @@
 #include "../../data_structure/glomconversions.h"
 #include "../../dialog_invalid_data.h"
 #include "../../application.h"
+#include "../../utils.h"
 //#include "../cellrendererlist.h"
 #include <iostream> //For debug output.
 
@@ -73,7 +74,10 @@ DbAddDel::DbAddDel()
   //set_columns_count(1);
   //construct_specified_columns();
 
+  m_ScrolledWindow.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
   m_ScrolledWindow.add(m_TreeView);
+  m_ScrolledWindow.set_shadow_type(Gtk::SHADOW_IN);
+
   m_TreeView.show();
   //m_TreeView.set_fixed_height_mode(); //This allows some optimizations.
   pack_start(m_ScrolledWindow);
@@ -104,12 +108,6 @@ DbAddDel::DbAddDel()
 
 DbAddDel::~DbAddDel()
 {
-  //if(m_modelcolumn_key)
-  //  delete m_modelcolumn_key;
-
-  //if(m_modelcolumn_placeholder)
-  //  delete m_modelcolumn_placeholder;
-
   App_Glom* pApp = get_application();
   if(pApp)
   {
@@ -346,7 +344,6 @@ Gtk::TreeModel::iterator DbAddDel::add_item(const Gnome::Gda::Value& valKey)
     Gtk::TreeModel::Row treerow = *result;
     if(treerow)
     {
-      treerow[*m_modelcolumn_key] = valKey;
       set_value_key(result, valKey);
       m_refListStore->set_is_placeholder(result, false);
       //treerow[*m_modelcolumn_placeholder] =  false; 
@@ -581,21 +578,6 @@ void DbAddDel::construct_specified_columns()
 
   Gtk::TreeModel::ColumnRecord record;
 
-  //Hidden internal columns:
-
-  //We must recreate these standard modelcolumns, because we can not reuse them
-  //if(m_modelcolumn_key)
-  //  delete m_modelcolumn_key;
-
-  //m_modelcolumn_key = new Gtk::TreeModelColumn<Gnome::Gda::Value>;
-
-  //if(m_modelcolumn_placeholder)
-  //  delete m_modelcolumn_placeholder;
-  //m_modelcolumn_placeholder = new Gtk::TreeModelColumn<bool>;
-
-  //record.add(*m_modelcolumn_key);
-  //record.add(*m_modelcolumn_placeholder);
-
   //Database columns:
   type_model_store::type_vec_fields fields;
   {
@@ -609,7 +591,7 @@ void DbAddDel::construct_specified_columns()
 
       record.add( *pModelColumn );
 
-      fields.push_back(iter->m_field);
+      fields.push_back(sharedptr<LayoutItem_Field>(new LayoutItem_Field(iter->m_field)));
 
       i++;
     }
@@ -620,7 +602,8 @@ void DbAddDel::construct_specified_columns()
   bool key_found = false;
   for(type_model_store::type_vec_fields::const_iterator iter = fields.begin(); iter != fields.end(); ++iter)
   {
-    if( !(iter->get_has_relationship_name()) && (iter->m_field.get_primary_key()) )
+    sharedptr<LayoutItem_Field> layout_item = *iter;
+    if( !(layout_item->get_has_relationship_name()) && (layout_item->m_field.get_primary_key()) )
     {
       key_found = true;
       break;
@@ -631,11 +614,11 @@ void DbAddDel::construct_specified_columns()
 
   if(!key_found)
   {
-    g_warning("DbAddDel::construct_specified_columns(): no primary key field found in:");
-    for(type_model_store::type_vec_fields::const_iterator iter = fields.begin(); iter != fields.end(); ++iter)
-    {
-      g_warning("  field: %s", iter->get_name().c_str());
-    }
+    g_warning("DbAddDel::construct_specified_columns(): no primary key field found.");
+    //for(type_model_store::type_vec_fields::const_iterator iter = fields.begin(); iter != fields.end(); ++iter)
+    //{
+    //  g_warning("  field: %s", (iter->get_name().c_str());
+    //}
 
     g_assert(key_found); //crash here.
   }
@@ -659,42 +642,13 @@ void DbAddDel::construct_specified_columns()
     DbAddDelColumnInfo& column_info = m_ColumnTypes[model_column_index];
     if(column_info.m_visible && !(column_info.m_field.get_hidden())) //TODO: We shouldn't need both of these.
     {
-      const Glib::ustring column_name = column_info.m_field.m_field.get_title_or_name();
+      const Glib::ustring column_name = column_info.m_field.get_title_or_name();
       const Glib::ustring column_id = column_info.m_field.get_name();
 
       int cols_count = 0;
       Gtk::CellRenderer* pCellRenderer = 0;
       switch(column_info.m_field.m_field.get_glom_type())
       {
-  /*
-        case(DbAddDelColumnInfo::STYLE_Choices):
-        {
-          //Use a custom CellRenderer:
-          CellRendererList* pCellRendererDerived = Gtk::manage( new CellRendererList() );
-
-          //Add the choices:
-          const type_vecStrings vecStrings = column_info.m_choices;
-          for(type_vecStrings::const_iterator iter = vecStrings.begin(); iter != vecStrings.end(); ++iter)
-          {
-            pCellRendererDerived->append_list_item(*iter);
-          }
-
-          pCellRenderer = pCellRendererDerived;
-
-          break;
-        }
-
-   */
-
-    /* Possible types:
-    TYPE_INVALID,
-    TYPE_NUMERIC,
-    TYPE_TEXT,
-    TYPE_DATE,
-    TYPE_TIME,
-    TYPE_BOOLEAN
-    */
-
         case(Field::TYPE_BOOLEAN):
         {
           pCellRenderer = Gtk::manage( new Gtk::CellRendererToggle() );
@@ -730,6 +684,9 @@ void DbAddDel::construct_specified_columns()
           //Make it editable:
           pCellRenderer->property_editable() = true;
 
+          if( column_info.m_field.m_field.get_glom_type() == Field::TYPE_NUMERIC )
+            pCellRenderer->property_xalign() = 1.0; //Align right.
+
           //Connect to its signal:
           pCellRenderer->signal_edited().connect(
             sigc::bind( sigc::mem_fun(*this, &DbAddDel::on_treeview_cell_edited), model_column_index) );
@@ -778,8 +735,8 @@ void DbAddDel::construct_specified_columns()
                   }
                 }
 
-                type_list_values_with_second list_values = get_choice_values(column_info.m_field);
-                for(type_list_values_with_second::const_iterator iter = list_values.begin(); iter != list_values.end(); ++iter)
+                GlomUtils::type_list_values_with_second list_values = GlomUtils::get_choice_values(column_info.m_field);
+                for(GlomUtils::type_list_values_with_second::const_iterator iter = list_values.begin(); iter != list_values.end(); ++iter)
                 {
                   const Glib::ustring first = GlomConversions::get_text_for_gda_value(column_info.m_field.m_field.get_glom_type(), iter->first, column_info.m_field.m_numeric_format);
 
@@ -828,6 +785,20 @@ void DbAddDel::construct_specified_columns()
   add_blank();
 }
 
+bool DbAddDel::refresh_from_database()
+{
+  if(m_refListStore)
+  {
+    Glib::RefPtr<Gtk::TreeModel> refNull;
+    bool result = m_refListStore->refresh_from_database(m_where_clause);
+    m_TreeView.set_model(refNull); //TODO: Find a better way to indicate that the whole model has changed, because this causes a g_waring().
+    m_TreeView.set_model(m_refListStore);
+    return result;
+  }
+  else
+    return false;
+}
+
 void DbAddDel::set_value(const Gtk::TreeModel::iterator& iter, const LayoutItem_Field& layout_item, const Gnome::Gda::Value& value)
 {
   //g_warning("DbAddDel::set_value begin");
@@ -1604,8 +1575,6 @@ void DbAddDel::set_value_key(const Gtk::TreeModel::iterator& iter, const Gnome::
     }
 
     m_refListStore->set_key_value(iter, value);
-
-   // row[*m_modelcolumn_key] = value;
   }
 }
 
